==> nanowire_flask-0.0.55-py3-none-any.whl: nanowire_flask/__init__.py <==
#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Jan 23 13:12:51 2019

@author: stuart
"""

#nanowire flask plugin tool



import time
from PIL import Image

import requests

from io import BytesIO

import traceback

import jsonpickle
from flask import request, Response
from flask.views import View

from flask_api import FlaskAPI


#memory and cpu usage collection tools

import sys
import os
from datetime import datetime
import psutil
import threading

py_version = int(sys.version[0])


if py_version == 2:
    import Queue as queue
else:
    import queue


import inspect

import logging

logging.basicConfig(format='%(message)s')

logger = logging.getLogger('NANOWIRE_FLASK_TOOL')

logger.setLevel(level=logging.DEBUG)


##############################
### ADVANCED LOGGING TOOLS ###
##############################

class usage_collection(object):
    
    def __init__(self):
        
        #times are collected as a series of strings
        self.times_queue = queue.Queue()
        #CPUs are collected in % usage
        self.cpus_queue = queue.Queue()
        #memory queue is in MB
        self.mem_queue = queue.Queue()
        self.running = False
    
    def collection_thread(self):
        
        self.running = True
        
        while self.running:
            
            self.times_queue.put_nowait(str(datetime.now().time()))
            self.cpus_queue.put_nowait(psutil.cpu_percent())
            self.mem_queue.put_nowait(psutil.Process(os.getpid()).memory_info().rss/1e6)
            time.sleep(0.01)
            logger.info("Background thread is running")
            
            
    def finish_collection(self):
        
        self.running = False

        
        times = []
        cpus = []
        mems = []
        while True:
            try:
                times.append(self.times_queue.get_nowait())
                cpus.append(self.cpus_queue.get_nowait())
                mems.append(self.cpus_queue.get_nowait())

            except:
                break
            
        max_mem = max(mems)
        max_cpu = max(cpus)
        
        return [max_mem, max_cpu]
    
    def start_collection(self):
        
        threading.Thread(target=self.collection_thread).start()


##############################
### IMAGE PROCESSING TOOLS ###
##############################

def run_image(r, app): 
 
    start_time = time.time()

    #if the user has sent a url then we want to extract that URL like this
    if r.headers['Content-Type'] != 'application/json':
    
        variables_info = dict(r.args)
                
        # convert string of image data to uint8
        im = Image.open(r.files['image']).convert("RGB")

    #if the user has sent an image then lets extract that image and store it as
    #a PIL
    else:
        #extract the variables info sent to the plugin
        variables_info = r.json

        #extract the image from the sent url
        im_request = requests.get(variables_info['contentUrl'])

        im = Image.open(BytesIO(im_request.content)).convert("RGB")
        
        variables_info.pop('contentUrl', None)

    #apply the function to the image
    
    if inspect.getargspec(app.config['function'])[0][-1] == 'variables':
        out_predictions = app.config['function'](im, variables_info)
    else:
        out_predictions = app.config['function'](im)

    
    print("Took {0:0.2f} seconds".format(time.time()-start_time))
    
    return out_predictions

#function to check if the user defined function is as it should be
def check_Image_function_is_valid(function):

    args = inspect.getargspec(function)[0]

    if args != ['img', 'variables'] and args != ['self', 'img', 'variables'] and args != ['img'] and args != ['self', 'img']:
        
        return False
        
    else:
        return True


class mount_Image_function(object):
    
    def __init__(self, function, debug_mode=False, host='0.0.0.0', port=5000, path='/model/predict'):
        
        self.app = FlaskAPI(__name__)

        self.app.config['debug'] = debug_mode
        
        self.app.config['function'] = function
        
        self.host = host
        
        self.port = port
        
        self.path = path
        
        if check_Image_function_is_valid(self.app.config['function']):
            
            #define the class we're mounting onto post
            tool = ImagesAPI
            
            #store a link to the app in tool
            tool.app = self.app
            
            #create the rule for /model/predict
            self.app.add_url_rule(self.path, view_func = tool.as_view('UserAPI'))
            
            #run the app
            self.app.run(host=self.host, port=self.port, debug=self.app.config['debug'])
        
        else:
            #if the function has bad inputs reject it
            raise Exception("BAD ARGUMENTS SENT TO FUNCTION")

    
class ImagesAPI(View):
    
    methods = ['POST']
    
    def dispatch_request(self):
        
        try:
            answer = run_image(request, self.app)
            
            if not isinstance(answer, dict):
                
                raise Exception("PLUGIN MUST RETURN A DICTIONARY, RETURNED {0}".format(str(type(answer))))
            
            answer['status'] = 'ok'
            
            response_pic = jsonpickle.encode(answer)
            #everything has gone fine, return the results in a nice response
            return Response(response=response_pic, status=200, mimetype="application/json")
        
        except Exception as exp:
            

            response = {'status':'failure', 'error':str(exp)}
            
            #if we're in debug mode then return a full traceback
            if self.app.config['debug']:
                response['full_traceback'] = str(traceback.format_exc())
                
            response_pic = jsonpickle.encode(response)
            
            #something has gone wrong return the bad result in this response
            return Response(response=response_pic, status=400, mimetype='application/json')


###########################################
### Functions for running a text plugin ###
###########################################
    

def run_text(r, app): 
 
    start_time = time.time()

    #if the user has sent a url then we want to extract that URL like this
    if r.headers['Content-Type'] != 'application/json':
    
        variables_info = dict(r.args)
        # convert string of image data to uint8
        text = r.files['doc'].read().decode()

    #if the user has sent an image then lets extract that image and store it as
    #a PIL
    else:
        #extract the variables info sent to the plugin
        variables_info = r.json
        
        #make sure content url is the right case
        #variables_info = map_contenturl2casecorrect(variables_info)

        if 'content' in variables_info.keys():

            #extract the image from the sent url
            text = variables_info['content']
            
            variables_info.pop('content', None)
            
        elif 'contentUrl' in variables_info.keys():
            
            response = requests.get(variables_info['contentUrl'])
                     
            text = response.content.decode()
            
            if response.status_code == 404:
                
                raise Exception("FILE MISSING, CHECK URL OF LINK")
            
            variables_info.pop('contentUrl', None)
            
        else:
            raise Exception("COULD NOT FIND 'contentUrl' OR 'content' IN REQUEST")

    #apply the function to the image
    if inspect.getargspec(app.config['function'])[0][-1] == 'variables':
        out_predictions = app.config['function'](text, variables_info)
    else:
        out_predictions = app.config['function'](text)
    
    print("Took {0:0.2f} seconds".format(time.time()-start_time))
    
    return out_predictions

#function to check if the user defined function is as it should be
def check_text_function_is_valid(function):

    args = inspect.getargspec(function)[0]

    if args != ['text', 'variables'] and args != ['self', 'text', 'variables'] and args != ['text'] and args != ['self', 'text']:
        
        return False
        
    else:
        return True


class mount_text_function(object):
    
    def __init__(self, function, debug_mode=False, host='0.0.0.0', port=5000, path='/model/predict'):
        
        self.app = FlaskAPI(__name__)

        self.app.config['debug'] = debug_mode
        
        self.app.config['function'] = function
        
        self.host = host
        
        self.port = port
        
        self.path = path
        
        if check_text_function_is_valid(self.app.config['function']):
            
            #define the class we're mounting onto post
            tool = TextAPI
            
            #if we're in debug mode we will need to collect usage statistics
            if debug_mode:
                tool.debug_mode = True
                tool.collection_tool = usage_collection()
            else:
                tool.debug_mode = False
            #store a link to the app in tool
            tool.app = self.app
            
            #create the rule for /model/predict
            self.app.add_url_rule(self.path, view_func = tool.as_view('UserAPI'))
            
            #run the app
            self.app.run(host=self.host, port=self.port, debug=self.app.config['debug'])
        
        else:
            #if the function has bad inputs reject it
            raise Exception("BAD ARGUMENTS SENT TO FUNCTION")

    
class TextAPI(View):
    
    #this is a post method
    methods = ['POST']
    
    
    
    def dispatch_request(self):
        
        try:
            
            if self.debug_mode:
                self.collection_tool.start_collection()
            #run the mounted function
            answer = run_text(request, self.app)
            
            if self.debug_mode:
                [max_cpu, max_mem] = self.collection_tool.finish_collection()
            
            if not isinstance(answer, dict):
                
                raise Exception("PLUGIN MUST RETURN A DICTIONARY, RETURNED {0}".format(str(type(answer))))
            
            answer['status'] = 'ok'
            
            if self.debug_mode:
                
                answer['max_cpu'] = max_cpu
                answer['max_mem'] = max_mem
            
            response_pic = jsonpickle.encode(answer)
            #everything has gone fine, return the results in a nice response
            return Response(response=response_pic, status=200, mimetype="application/json")
        
        except Exception as exp:
            
            response = {'status':'failure', 'error':str(exp)}
            
            #if we're in debug mode then return a full traceback
            if self.app.config['debug']:
                response['full_traceback'] = str(traceback.format_exc())
                
            response_pic = jsonpickle.encode(response)
            
            #something has gone wrong return the bad result in this response
            return Response(response=response_pic, status=400, mimetype='application/json')

==> nanowire_flask-0.0.56-py3-none-any.whl: nanowire_flask/__init__.py <==
#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Jan 23 13:12:51 2019

@author: stuart
"""

#nanowire flask plugin tool



import time
from PIL import Image

import requests

from io import BytesIO

import traceback

import jsonpickle
from flask import request, Response
from flask.views import View

from flask_api import FlaskAPI


#memory and cpu usage collection tools

import sys
import os
from datetime import datetime
import psutil
import threading

py_version = int(sys.version[0])


if py_version == 2:
    import Queue as queue
else:
    import queue


import inspect

import logging

logging.basicConfig(format='%(message)s')

logger = logging.getLogger('NANOWIRE_FLASK_TOOL')

logger.setLevel(level=logging.DEBUG)


##############################
### ADVANCED LOGGING TOOLS ###
##############################

class usage_collection(object):
    
    def __init__(self):
        
        #times are collected as a series of strings
        self.times_queue = queue.Queue()
        #CPUs are collected in % usage
        self.cpus_queue = queue.Queue()
        #memory queue is in MB
        self.mem_queue = queue.Queue()
        self.running = False
    
    def collection_thread(self):
        
        self.running = True
        
        while self.running:
            
            self.times_queue.put_nowait(str(datetime.now().time()))
            self.cpus_queue.put_nowait(psutil.cpu_percent())
            self.mem_queue.put_nowait(psutil.Process(os.getpid()).memory_info().rss/1e6)
            time.sleep(0.01)
            logger.info("Background thread is running")
            
            
    def finish_collection(self):
        
        self.running = False

        
        times = []
        cpus = []
        mems = []
        while True:
            try:
                times.append(self.times_queue.get_nowait())
                cpus.append(self.cpus_queue.get_nowait())
                mems.append(self.mem_queue.get_nowait())

            except:
                break
            
        max_mem = max(mems)
        max_cpu = max(cpus)
        
        return [max_mem, max_cpu]
    
    def start_collection(self):
        
        threading.Thread(target=self.collection_thread).start()


##############################
### IMAGE PROCESSING TOOLS ###
##############################

def run_image(r, app): 
 
    start_time = time.time()

    #if the user has sent a url then we want to extract that URL like this
    if r.headers['Content-Type'] != 'application/json':
    
        variables_info = dict(r.args)
                
        # convert string of image data to uint8
        im = Image.open(r.files['image']).convert("RGB")

    #if the user has sent an image then lets extract that image and store it as
    #a PIL
    else:
        #extract the variables info sent to the plugin
        variables_info = r.json

        #extract the image from the sent url
        im_request = requests.get(variables_info['contentUrl'])

        im = Image.open(BytesIO(im_request.content)).convert("RGB")
        
        variables_info.pop('contentUrl', None)

    #apply the function to the image
    
    if inspect.getargspec(app.config['function'])[0][-1] == 'variables':
        out_predictions = app.config['function'](im, variables_info)
    else:
        out_predictions = app.config['function'](im)

    
    print("Took {0:0.2f} seconds".format(time.time()-start_time))
    
    return out_predictions

#function to check if the user defined function is as it should be
def check_Image_function_is_valid(function):

    args = inspect.getargspec(function)[0]

    if args != ['img', 'variables'] and args != ['self', 'img', 'variables'] and args != ['img'] and args != ['self', 'img']:
        
        return False
        
    else:
        return True


class mount_Image_function(object):
    
    def __init__(self, function, debug_mode=False, host='0.0.0.0', port=5000, path='/model/predict'):
        
        self.app = FlaskAPI(__name__)

        self.app.config['debug'] = debug_mode
        
        self.app.config['function'] = function
        
        self.host = host
        
        self.port = port
        
        self.path = path
        
        if check_Image_function_is_valid(self.app.config['function']):
            
            #define the class we're mounting onto post
            tool = ImagesAPI
            
            #store a link to the app in tool
            tool.app = self.app
            
            #create the rule for /model/predict
            self.app.add_url_rule(self.path, view_func = tool.as_view('UserAPI'))
            
            #run the app
            self.app.run(host=self.host, port=self.port, debug=self.app.config['debug'])
        
        else:
            #if the function has bad inputs reject it
            raise Exception("BAD ARGUMENTS SENT TO FUNCTION")

    
class ImagesAPI(View):
    
    methods = ['POST']
    
    def dispatch_request(self):
        
        try:
            answer = run_image(request, self.app)
            
            if not isinstance(answer, dict):
                
                raise Exception("PLUGIN MUST RETURN A DICTIONARY, RETURNED {0}".format(str(type(answer))))
            
            answer['status'] = 'ok'
            
            response_pic = jsonpickle.encode(answer)
            #everything has gone fine, return the results in a nice response
            return Response(response=response_pic, status=200, mimetype="application/json")
        
        except Exception as exp:
            

            response = {'status':'failure', 'error':str(exp)}
            
            #if we're in debug mode then return a full traceback
            if self.app.config['debug']:
                response['full_traceback'] = str(traceback.format_exc())
                
            response_pic = jsonpickle.encode(response)
            
            #something has gone wrong return the bad result in this response
            return Response(response=response_pic, status=400, mimetype='application/json')


###########################################
### Functions for running a text plugin ###
###########################################
    

def run_text(r, app): 
 
    start_time = time.time()

    #if the user has sent a url then we want to extract that URL like this
    if r.headers['Content-Type'] != 'application/json':
    
        variables_info = dict(r.args)
        # convert string of image data to uint8
        text = r.files['doc'].read().decode()

    #if the user has sent an image then lets extract that image and store it as
    #a PIL
    else:
        #extract the variables info sent to the plugin
        variables_info = r.json
        
        #make sure content url is the right case
        #variables_info = map_contenturl2casecorrect(variables_info)

        if 'content' in variables_info.keys():

            #extract the image from the sent url
            text = variables_info['content']
            
            variables_info.pop('content', None)
            
        elif 'contentUrl' in variables_info.keys():
            
            response = requests.get(variables_info['contentUrl'])
                     
            text = response.content.decode()
            
            if response.status_code == 404:
                
                raise Exception("FILE MISSING, CHECK URL OF LINK")
            
            variables_info.pop('contentUrl', None)
            
        else:
            raise Exception("COULD NOT FIND 'contentUrl' OR 'content' IN REQUEST")

    #apply the function to the image
    if inspect.getargspec(app.config['function'])[0][-1] == 'variables':
        out_predictions = app.config['function'](text, variables_info)
    else:
        out_predictions = app.config['function'](text)
    
    print("Took {0:0.2f} seconds".format(time.time()-start_time))
    
    return out_predictions

#function to check if the user defined function is as it should be
def check_text_function_is_valid(function):

    args = inspect.getargspec(function)[0]

    if args != ['text', 'variables'] and args != ['self', 'text', 'variables'] and args != ['text'] and args != ['self', 'text']:
        
        return False
        
    else:
        return True


class mount_text_function(object):
    
    def __init__(self, function, debug_mode=False, host='0.0.0.0', port=5000, path='/model/predict'):
        
        self.app = FlaskAPI(__name__)

        self.app.config['debug'] = debug_mode
        
        self.app.config['function'] = function
        
        self.host = host
        
        self.port = port
        
        self.path = path
        
        if check_text_function_is_valid(self.app.config['function']):
            
            #define the class we're mounting onto post
            tool = TextAPI
            
            #if we're in debug mode we will need to collect usage statistics
            if debug_mode:
                tool.debug_mode = True
                tool.collection_tool = usage_collection()
            else:
                tool.debug_mode = False
            #store a link to the app in tool
            tool.app = self.app
            
            #create the rule for /model/predict
            self.app.add_url_rule(self.path, view_func = tool.as_view('UserAPI'))
            
            #run the app
            self.app.run(host=self.host, port=self.port, debug=self.app.config['debug'])
        
        else:
            #if the function has bad inputs reject it
            raise Exception("BAD ARGUMENTS SENT TO FUNCTION")

    
class TextAPI(View):
    
    #this is a post method
    methods = ['POST']
    
    
    
    def dispatch_request(self):
        
        try:
            
            if self.debug_mode:
                self.collection_tool.start_collection()
            #run the mounted function
            answer = run_text(request, self.app)
            
            if self.debug_mode:
                [max_mem, max_cpu] = self.collection_tool.finish_collection()
            
            if not isinstance(answer, dict):
                
                raise Exception("PLUGIN MUST RETURN A DICTIONARY, RETURNED {0}".format(str(type(answer))))
            
            answer['status'] = 'ok'
            
            if self.debug_mode:
                
                answer['max_cpu'] = max_cpu
                answer['max_mem'] = max_mem
            
            response_pic = jsonpickle.encode(answer)
            #everything has gone fine, return the results in a nice response
            return Response(response=response_pic, status=200, mimetype="application/json")
        
        except Exception as exp:
            
            response = {'status':'failure', 'error':str(exp)}
            
            #if we're in debug mode then return a full traceback
            if self.app.config['debug']:
                response['full_traceback'] = str(traceback.format_exc())
                
            response_pic = jsonpickle.encode(response)
            
            #something has gone wrong return the bad result in this response
            return Response(response=response_pic, status=400, mimetype='application/json')
